from base64 import b64encode
from datetime import datetime

import httpx

from mode_discovery_api.models import SignatureToken


class ApiClient(object):
    def __init__(self, workspace: str, token: str, password: str, host: str = 'https://app.mode.com/batch'):
        self.auth = httpx.BasicAuth(token, password)
        self.base_url = f"{host}/{workspace}"

    @staticmethod
    def _get_bearer_token(signature_token: SignatureToken) -> str:
        decoded_token = f"{signature_token.token}:{signature_token.access_key}:{signature_token.access_secret}"
        encoded_token = b64encode(decoded_token.encode('utf-8'))
        bearer_token = 'Bearer ' + encoded_token.decode('utf-8')
        return bearer_token

    def create_signature_token(self, name: str, expires_at: datetime,
                               authentication_for: str = 'batch-api',
                               authorization_type: str = 'read-only') -> SignatureToken:
        data = {
            "signature_token": {
                "name": name,
                "expires_at": expires_at.isoformat(),
                "auth_scope": {"authentication_for": authentication_for, "authorization_type": authorization_type}
            }
        }

        r = httpx.post(f"{self.base_url}/signature_tokens", json=data, auth=self.auth)
        r.raise_for_status()

        return SignatureToken.parse_obj(r.json())

    def delete_signature_token(self, signature_token: str):
        r = httpx.delete(f"{self.base_url}/signature_tokens/{signature_token}", auth=self.auth)
        r.raise_for_status()

        return r
